@@ -1,10 +1,9 @@
 import binascii
 import random
+import redis
 import time
 import typing
 
-import redis
-
 from . import _connector
 
 
@@ -32,8 +31,8 @@ class Lock(
 
         while True:
             if self.redis_connection.set(
-                self.name,
-                b'',
+                name=self.name,
+                value=b'',
                 nx=True,
                 ex=ttl,
             ):
@@ -98,9 +97,9 @@ class QueueRedis(
 ):
     def __init__(
         self,
-        *args,
-        **kwargs,
-    ):
+        *args: typing.Any,
+        **kwargs: typing.Any,
+    ) -> None:
         super().__init__(*args, **kwargs)
 
         self.delayed_queue_pop_bulk_script = self.register_script(
@@ -122,7 +121,7 @@ class QueueRedis(
         self,
         queue_name: str,
         include_delayed: bool,
-    ):
+    ) -> int:
         pipeline = self.pipeline()
         pipeline.llen(
             name=queue_name,
@@ -144,7 +143,7 @@ class QueueRedis(
     def queue_delete(
         self,
         queue_name: str,
-    ):
+    ) -> int:
         return self.delete(
             queue_name,
             f'{queue_name}.delayed',
@@ -153,7 +152,7 @@ class QueueRedis(
     def queue_pop(
         self,
         queue_name: str,
-    ):
+    ) -> typing.Optional[typing.Any]:
         item = self.lpop(
             name=queue_name,
         )
@@ -185,7 +184,7 @@ class QueueRedis(
         self,
         queue_name: str,
         number_of_items: int,
-    ):
+    ) -> typing.List[typing.Any]:
         pipeline = self.pipeline()
         pipeline.lrange(queue_name, 0, number_of_items - 1)
         pipeline.ltrim(queue_name, number_of_items, -1)
